package octoshape.android.mediaplayer;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;

import com.google.android.gms.cast.ApplicationMetadata;
import com.google.android.gms.cast.MediaInfo;
import octoshape.android.CastApplication;
import octoshape.android.R;
import com.google.sample.cast.refplayer.utils.Utils;
import com.google.android.libraries.cast.companionlibrary.cast.VideoCastManager;
import com.google.android.libraries.cast.companionlibrary.cast.callbacks.VideoCastConsumerImpl;

import octoshape.android.stream.StreamManager;

/**
 * Handles playing locally or remotely. In case the app is connected to a Google Cast device,
 * it will play on the Google Cast. Otherwise, it will play localy. Also, it handles switching from
 * local to remote to local playing.
 * Created by dota on 19-02-2015.
 */
public class PlayerManager {

    public static final String TAG = "PlayerManager";

    private VideoCastManager castManager;
    private PlaybackLocation playbackLocation = PlaybackLocation.LOCAL;
    private VideoCastConsumerImpl castConsumer;

    private IMediaPlayer mediaPlayer;
    private Context context;

    private MediaInfo mediaInfo;
    private String stream;
    private Activity activity;

    /*
  * indicates whether we are doing a local or a remote playback
  */
    public enum PlaybackLocation {
        LOCAL,
        REMOTE;
    }

    private PlayerManager(Context c) {
        castManager = CastApplication.getCastManager();
        this.context = c;

        setupCastListener();
        castManager.addVideoCastConsumer(castConsumer);
    }

    public static PlayerManager initializePlayerManager(Context c) {
        return new PlayerManager(c);
    }

    /**
     * Adding a cast listener. In case we are connecte
     */
    private void setupCastListener() {
        castConsumer = new VideoCastConsumerImpl() {
            public static final String TAG = "lpa_RemotePlayer";

            @Override
            public void onApplicationConnected(ApplicationMetadata appMetadata,
                                               String sessionId, boolean wasLaunched) {
                Log.d(TAG, "onApplicationLaunched() is reached");

                updatePlaybackLocation(PlaybackLocation.REMOTE);
            }

            @Override
            public void onApplicationDisconnected(int errorCode) {
                Log.d(TAG, "onApplicationDisconnected() is reached with errorCode: " + errorCode);
                updatePlaybackLocation(PlaybackLocation.LOCAL);
            }

            @Override
            public void onDisconnected() {
                Log.d(TAG, "onDisconnected() is reached");
                updatePlaybackLocation(PlaybackLocation.LOCAL);
            }

            @Override
            public void onRemoteMediaPlayerMetadataUpdated() {
            }

            @Override
            public void onFailed(int resourceId, int statusCode) {
                Log.d(TAG, "onFailed() is reached");
            }

            @Override
            public void onConnectionSuspended(int cause) {
                Log.d(TAG, "onConnectionSuspended() is reached");
                Utils.showToast(context,
                        R.string.connection_temp_lost);
            }

            @Override
            public void onConnectivityRecovered() {
                Utils.showToast(context,
                        R.string.connection_recovered);
            }

            @Override
            public void onApplicationStatusChanged(String appStatus) {
                Log.d(TAG, "Application status changed: " + appStatus);
            }
        };
    }

    /**
     * Updates the playback location and stops playback on the previous player.
     * @param playbackLocation
     */
    private void updatePlaybackLocation(PlaybackLocation playbackLocation) {
        this.playbackLocation = playbackLocation;
        int playbackPosition = 0;
        IMediaPlayer.PlaybackState playbackState = IMediaPlayer.PlaybackState.PAUSED;

        if (mediaPlayer != null) {
            playbackPosition = mediaPlayer.getPlaybackPosition();
            playbackState = mediaPlayer.getPlaybackState();
            mediaPlayer.stopPlay();
        }

        boolean playing = false;
        if (playbackState == IMediaPlayer.PlaybackState.PLAYING) {
            playing = true;
        }
        
        // if live always make sure that we start at the beginning
        if(mediaInfo != null && mediaInfo.getStreamType() == MediaInfo.STREAM_TYPE_LIVE)
            playbackPosition = 0;

        chooseLocation(playbackPosition, playing);
    }

    /**
     * Starts playing the stream with the provided media info
     * @param activity the Activity calling this method
     * @param mediaInfoBundle the associated MediaInfo
     */
    public void startPlaying(Activity activity, Bundle mediaInfoBundle) {
        this.activity = activity;
        MediaInfo _mediaInfo = com.google.android.libraries.cast.companionlibrary.utils.Utils.bundleToMediaInfo(mediaInfoBundle);
        String _stream = StreamManager.getStreamFromMediaInfo(_mediaInfo);
        // in case startPlaying is called twice for the same stream
        if (mediaPlayer != null && this.stream != null && this.stream.equals(_stream)) {
            Log.d(TAG, "Double firing!");
            return;
        }

        if (mediaPlayer != null) {
            mediaPlayer.stopPlay();
        }

        this.mediaInfo = _mediaInfo;
        this.stream = _stream;

        chooseLocation(0, true);
    }


    private void chooseLocation(int position, boolean play) {
        Log.d(TAG, "Choose new location, position: " + position);
        switch (playbackLocation) {
            case LOCAL:
                playLocally(mediaInfo, position, play);
                break;
            case REMOTE:
                playRemotely(mediaInfo, position, play);
                break;
            default:
        }
    }

    private void playLocally(MediaInfo mediaInfo, int position, boolean play) {
        if (mediaInfo == null) {
            Log.d(TAG, "Local media player chosen, but mediaInfo is null");
        } else {
            Log.d(TAG, "Will play with local media player. [" + position + "]");
            Intent intent = new Intent(context, LocalPlayerActivity.class);
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            intent.putExtra("media", com.google.android.libraries.cast.companionlibrary.utils.Utils.mediaInfoToBundle(mediaInfo));
            intent.putExtra("shouldStart", play);
            intent.putExtra("position", position);
            context.startActivity(intent);
        }
    }

    private void playRemotely(MediaInfo mediaInfo, int position, boolean play) {
        if (mediaInfo != null) {
            Log.d(TAG, "Will play with remote media player.");
            RemotePlayerActivity rpa = new RemotePlayerActivity(stream, mediaInfo);
            mediaPlayer = rpa;
            if (activity == null) {
                rpa.startPlaying(context, position, play);
            } else {
                rpa.startPlaying(activity, position, play);
            }
        } else {
            Log.d(TAG, "Remote media player chosen, but mediaInfo is null");
        }
    }

    /**
     * Updates the media info, only if the media info is for the currently playing stream
     * @param stream
     * @param mediaInfo
     */
    public void updatedMediaInfo(String stream, MediaInfo mediaInfo) {

        Log.d(TAG, "Updated MediaInfo for " + stream);

        if (this.stream != null && this.stream.equals(stream) && mediaPlayer != null) {
            this.mediaInfo = mediaInfo;
            this.mediaPlayer.updatedMediaInfo(mediaInfo);
        }
    }

    public void updateDuration(String stream, long duration, MediaInfo mediaInfo) {

        if (this.stream != null && this.stream.equals(stream)) {
            this.mediaInfo = mediaInfo;
            mediaPlayer.updateDuration(duration);
        }
    }

    public void setMediaPlayer(IMediaPlayer player) {
        this.mediaPlayer = player;
    }


    public void stopPlaying() {
        if (mediaPlayer != null) {
            mediaPlayer.stopPlay();
        }
    }
}
